//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
#include <cpctelera.h>
#include <stdbool.h>
#include <stdio.h>
#include "ai.h"
#include "consts.h"
#include "entity.h"
#include "aux_math.h"
#include "components/fighter.h"

#ifdef DEBUG
  #include <assert.h>
#endif

struct TAI ai_components[MAX_AI_COMPONENTS];
u8 num_ai_components;

//---------------------------------------------------------------------------
void init_ai() {
  cpct_memset (ai_components, 0x0, sizeof (ai_components));
  num_ai_components = 0;
}

//---------------------------------------------------------------------------
struct TAI *basic_ai_create ()
{
  struct TAI *ai = NULL;

  if (num_ai_components < MAX_AI_COMPONENTS)
  {

#ifdef DEBUG
    assert (num_ai_components < MAX_AI_COMPONENTS);
#endif
    // Get an empty slot from the ai array
    ai = &ai_components[num_ai_components++];

    // Set the AI TakeTurn function
    ai->take_turn = basic_ai_take_turn;

    // Return the ai ptr as our new allocated ai
  }

#ifdef DEBUG
    assert (ai);
#endif

  return ai;

}
//---------------------------------------------------------------------------
/*
void test_ai_take_turn (struct TAI *ai)
{
  i8 dx, dy;
  u8 rnd_val = cpct_rand();
  if (rnd_val < 128)
    dx = -1;
  else if (rnd_val > 140)
    dx = 1;
  else
    dx = 0;

  rnd_val = cpct_rand();
  if (rnd_val < 128)
    dy = -1;
  else if (rnd_val > 140)
    dy = 1;
  else
    dy = 0;
  entity_move (ai->owner, dx, dy);
}
*/
//---------------------------------------------------------------------------
void basic_ai_take_turn (struct TAI *ai)
{
  extern struct TEntity *player;
  struct TEntity *ai_entity = ai->owner;
  u16 distance = distance_sqr(player->x, player->y, ai_entity->x, ai_entity->y);

  // If enemy is in player's view radius wake up
  if (distance < MOB_ATKR2) {
    // chase player if farther than 1 tiles
    if (distance >= 2)
      move_to (ai_entity, player->x, player->y);

    // if mob is next to player attack!
    else if (player->fighter->hp > 0) {
      fighter_attack (ai_entity->fighter, player);
    } // if
  } // if
  // Out of player view, just do nothing for now
}

bool move_to (struct TEntity *e, i8 x, i8 y)
{
  i8 dx, dy;
  dx = dy = 0;

  if (e->x != x) {
    // Horizontal
    dx = sign (x - e->x);
  }
  if (e->y != y) {
    dy = sign (y - e->y);
  }
  entity_move (e, dx, dy);
  return (dx == 0 && dy == 0);
}