//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------

#include <components/weapon.h>
// forward declaration of TEntity
typedef struct TEntity;
// forward declaration of TDamage
typedef struct TDamage;

///< Fighter component
struct TFighter{
  struct TEntity *owner;  ///< Entity that owns this component
  i8 max_hp;              ///< Max health
  i8 hp;                  ///< Current health
  u8 strn;                ///< Strength
  u8 def;                 ///< Defense
  u8 ref;                 ///< Reflexes
  u8 dex;                 ///< Dexterity
  u16 xp;                 ///< Experience given when killed
  struct TDamage dmg;     ///< Base damage
  ///< Death fn. to call when entity dies
  void (*death_fn)(struct TEntity *);
};

///< Mob default stats structure
struct TFighterInfo {
  i8  hp;                 ///< Max health
  u16 xp;                 ///< Experience given when killed
  u8 strn;                ///< Strength
  u8 def;                 ///< Defense
  u8 ref;                 ///< Reflexes
  u8 dex;                 ///< Dexterity
  u8 min_dmg;             ///< Base damage (Min)
  u8 max_dmg;             ///< Base damage (Max)
};

extern u8 num_fighters;
extern const struct TFighterInfo mob_info[];



void init_fighters();
struct TFighter *fighter_create (
  i8  hp,
  u16 xp,
  u8  strn,
  u8  def,
  u8  ref,
  u8  dex,
  u8  min_dmg, u8 max_dmg,
  void (*death_fn) (struct TEntity *));

/*! \brief
 *
 * Take dmg points of damage
 *
 */
void fighter_take_damage (struct TFighter *fighter,
                          struct TEntity *attacker,
                          i8 dmg);

/*! \brief
 *
 *  Perform an attack action to an entity!
 *
 */
void fighter_attack (struct TFighter *fighter,
                     struct TEntity *target);

void kill_mob (struct TEntity *mob);
void kill_player(struct TEntity *e);