//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
#include <cpctelera.h>
#include <stdio.h>
#include <stdbool.h>
#include <string.h>
#include "conio.h"
#include "consts.h"
//#include <stdio.h>

//TConMsg con_messages[];
u8 num_messages;


void clr_log() {
  clrwin ((void*)VMEM_LOG, LOG_W, 5, DEFAULT_BG);
}

void clr_stats() {
  clrwin ((void*)VMEM_STATS_XP, STATS_W, 12, DEFAULT_BG);
}

void clr_map() {
  clrwin ((void*)VMEM_MAP, VMAP_WIDTH, VMAP_HEIGHT, DEFAULT_BG);
}

/*!
 * Starting at given memory address (usually video memory) it clears
 * the specified cols x rows section.
 *
 *  \param pvmem_start  Address of the start cell to clear
 *  \param cols         Width in bytes of the portion to clear
 *  \param rows         Height in characters, not bytes or pixels.
 *                      There are 25 rows in Mode 1
 *  \param color Color pattern to use as byte pattern, not pen color
 */
void clrwin (u8 *pvmem_start, u8 cols, u8 rows, u8 color)
{
  for (u8 i=0; i != rows; ++i) {
    for (u8 line=0; line != 8; ++line) {
      cpct_memset (pvmem_start+i*0x50+line*0x800, color, cols);
    }
  }
  num_messages = 0;
}

/*
 * It calls cpct_drawCharM1_f to draw a character very fast at a specified.
 * In addition, foreground and background colors are used to color the char.
 *
 * \param vmem Start of video memory buffer to draw
 * \param x,y  X,Y coordinates (in characters) where to draw
 * \param ch   Character to draw
 * \param fg_pen Foreground color to use to draw the char
 * \param bg_pen Background color to use to draw the char
 */
void putchar_f (void *pvmem_start, u8 x, u8 y, u8 ch, u8 fg_pen, u8 bg_pen)
{
  void *pvdest = cpct_getScreenPtr (pvmem_start, x*2, y*8);

  cpct_drawCharM1_f (pvdest, fg_pen, bg_pen, ch);
}
//---------------------------------------------------------------------------
void log_msg_ext (char *text) {
  char *pch;
  u8 fg, bg, col=0;

  pch=strtok (text,"\477");

  while (pch != NULL) {
    fg = *pch++;
    bg = *pch++;
    fg -= 8;
    bg -= 8;
    cpct_drawStringM1_f (pch, (void*)(VMEM_LOG+num_messages*0x50+col),
      fg, bg);
    col += strlen(pch)<<1;
    pch = strtok (NULL, "\477");
  }
  num_messages++;
  if (num_messages > 3) {
    cpct_drawStringM1_f ("Press [SPACE] to continue",
      (void*)(VMEM_LOG+num_messages*0x50),
      REVERSE_FG, REVERSE_BG);

    // Wait for Key pressed
    wait_for_key (Key_Space);

    // clear log window
    clr_log();
  }
}
//---------------------------------------------------------------------------
void press_any_key_cont() {
  void *pvmem = cpct_getScreenPtr (CPCT_VMEM_START, 5*2, 23*8);
  cpct_drawStringM1_f ("Press [SPACE] to continue", pvmem, 2,0);

  cpct_memset (cpct_keyboardStatusBuffer, 0, sizeof (cpct_keyboardStatusBuffer));

  // Wait for Key pressed
  wait_for_key (Key_Space);
}
//---------------------------------------------------------------------------
void wait_for_key(cpct_keyID key) {

  cpct_scanKeyboard();
  while (!cpct_isKeyPressed(key)) {
    cpct_scanKeyboard();
  }
}
//---------------------------------------------------------------------------
char *get_string (char *str, u8 x, u8 y, u8 fg_color, u8 bg_color)
{
  char ch, *retstr;
  retstr = str;

  while ( (ch = getchar()) != '\r') {
    *str++ = ch;
    putchar_f ((void*)0xC000, x++, y, ch, fg_color, bg_color);
  }
  *str = '\0';

  return retstr;
}
//---------------------------------------------------------------------------
void display_text_lines (TTextLine text_lines[], u8 nlines)
{
  TTextLine *plast_line  = &(text_lines[nlines]);
  TTextLine  *pline = text_lines;
  while (pline < plast_line) {
    cpct_drawStringM1_f (pline->text,
      cpct_getScreenPtr ((void*)0xC000, pline->x*2, pline->y*8),
      pline->fg_color,
      pline->bg_color);
    pline++;
  }
}
//---------------------------------------------------------------------------
u8 get_center (u8 startx, u8 width, char *str)
{
  return startx + width - strlen (str);
}
//---------------------------------------------------------------------------
void fw_set_palette (u8 acolors[])
{

  u8 *data = acolors;
  for (u8 i=0; i<4; i++) {
    putchar (0x1C);        // INK CMD
    putchar (*data++);  // PEN
    putchar (*data++);    // COLOR1
    putchar (*data++);  // COLOR2
  }
}
//---------------------------------------------------------------------------
void fw_set_border  (u8 bgcolor)
{
  putchar (0x1D);
  putchar (bgcolor);
  putchar (bgcolor);
}