//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
#ifndef CONST_H
#define CONST_H

///< Max allowed entities in game
#define MAX_ENTITIES            128

///< How many fighter components we allocate, by default we allocate as many
/// as max entities
#define MAX_FIGHTER_COMPONENTS  96

///< How many AI components we allocate, by default we allocate as many
/// as max entities
#define MAX_AI_COMPONENTS    96
#define MAX_ITEMS            96
#define MAX_STAIRS            2

// How many potions
#define MAX_POTIONS           6
#define POISON_DURATION      12
#define BONUS_DURATION       12
#define HUNGRY_COUNTER       255
#define ORUTRIX_MIN_DLVL      10
#define ORUTRIX_MAX_DLVL      14

///< How many lines of text in the credits
#define NUM_CREDITS_LINES     4

// Map related consts.
#define MAP_WIDTH            28
#define MAP_HEIGHT           18
#define GRID_WIDTH            3
#define GRID_HEIGHT           2
#define GRID_SIZE            GRID_WIDTH * GRID_HEIGHT
#define ROOM_NONE           255
#define GRID_CELL_WIDTH      MAP_WIDTH/GRID_WIDTH
#define GRID_CELL_HEIGHT     MAP_HEIGHT/GRID_HEIGHT
#define MIN_ROOMS             4
#define MAX_ROOMS            GRID_SIZE
#define MIN_ROOM_SIZE         5
#define MAX_ROOM_SIZE         8

// Max mobs/room
#define MAX_MOBS_PER_ROOM     2
#define MAX_ITEMS_PER_ROOM    1

/*
#define   PEN_ENTITY          3
#define   PEN_CLEAR           0
#define   PEN_TILE            3
#define   PEN_EXPLORED        1
#define   PEN_BRIGHT          2
#define   PEN_NORMAL          3
#define   PEN_MESSAGES        1
#define   PEN_GOLD            1
#define   PEN_POTION          2
*/
// HARDWARE COLORS
#define BLACK           0x14
#define BLUE            0x04
#define BRIGHT BLUE     0x15
#define RED             0x1C
#define MAGENTA         0x18
#define MAUVE           0x1D
#define BRIGHT_RED      0x0C
#define PURPLE          0x05
#define BRIGHT_MAGENTA  0x0D
#define GREEN           0x16
#define CYAN            0x06
#define SKY_BLUE        0x17
#define YELLOW          0x1E
#define WHITE           0x00
#define PASTEL_BLUE     0x1F
#define ORANGE          0x0E
#define PINK            0x07
#define PASTEL_MAGENTA  0x0F
#define BRIGHT_GREEN    0x12
#define SEA_GREEN       0x02
#define BRIGHT_CYAN     0x13
#define LIME            0x1A
#define PASTEL_GREEN    0x19
#define PASTEL_CYAN     0x1B
#define BRIGHT_YELLOW   0x0A
#define PASTEL_YELLOW   0x03
#define BRIGHT_WHITE    0x0B

// IN GAME COLORS
// 0: BLACK
// 1: PASTEL_YELLOW
// 2: RED
// 3: CYAN
#define DEFAULT_BG      0
#define DEFAULT_FG      1
#define REVERSE_BG      1
#define REVERSE_FG      0
#define WALL_FG         2
#define WARNING_FG      1
#define WARNING_BG      2
#define FLOOR_FG        3
#define INV_ITEM        DEFAULT_FG
#define INV_LETTER      3
#define STATS_FG        DEFAULT_FG


// Game Sprites
#define   SPR_VWALL         0x95
#define   SPR_HWALL         0x9A
#define   SPR_BLWALL        0x93
#define   SPR_TLWALL        0x96
#define   SPR_BRWALL        0x99
#define   SPR_TRWALL        0x9C
#define   SPR_FLOOR         0x90
#define   SPR_TUNNEL        0xCF
#define   SPR_PLAYER        '@'
#define   SPR_GOBLIN        'g'
#define   SPR_SNAKE         'S'
#define   SPR_KOBOLD        'K'
#define   SPR_ORC           'o'
#define   SPR_BAT           'B'
#define   SPR_REMAINS       '%'
#define   SPR_GOLD          '$'
#define   SPR_FOOD          0xE2
#define   SPR_POTION        175   // ¡
#define   SPR_STAIRSUP      0x3C  // <
#define   SPR_STAIRSDOWN    0x3E  // >
#define   SPR_ORUTRIX       0xB5

// Entity flags
#define   E_IN_WORLD        0x01
#define   E_DEAD            0x02
#define   E_POISONED        0x04
#define   E_STR_BONUS       0x08
#define   E_DEF_BONUS       0x10
#define   E_DEX_BONUS       0x20
#define   E_REF_BONUS       0x40
#define   E_STATS_IMPR      0x78
#define   E_HUNGRY          0x80

// Where the visible areas start, these offsets are added to
// everything we draw so they appear in the correct playable coordinates
#define   VIEW_X               1      // X map area display
#define   VIEW_Y               1      // Y map area display
#define   VMAP_WIDTH          28*2
#define   VMAP_HEIGHT         18
#define   LOG_W               38*2    // in bytes
#define   LOG_X                1      // X log messages area
#define   LOG_Y               20      // Y log messages area
#define   STATS_Y              2
#define   STATS_W             80-MAP_WIDTH*2-4

// Cell Flags
#define   BLOCKED           0x01  ///< Blocks movement
#define   TUNNEL            0x02  ///< Has been explored by player

// Inventory and items
#define   MAX_INVENTORY_ITEMS 16  ///< Inventory capacity

// Experience and level-ups
#define   LEVEL_UP_BASE       75
#define   LEVEL_UP_FACTOR     50

// LOS constants
#define   FOV_RADIUS           5          // Max distance player can see
#define   FOV_R2    FOV_RADIUS*FOV_RADIUS // Squared
#define   MOB_ATKR2         FOV_R2*2

//!< Address of different screen areas
#define VMEM              (0xC000)
#define VMEM_MAP          (0xC000+VIEW_Y*0x50+VIEW_X*2)
#define VMEM_LOG          (0xC000+ LOG_Y*0x50+LOG_X*2)
#define VMEM_STATS_XP     (0xC000+STATS_Y*0x50+(MAP_WIDTH+VIEW_X)*2)
#define VMEM_STATS_HP     (0xC000+(STATS_Y+1)*0x50+(MAP_WIDTH+VIEW_X)*2)
#define VMEM_STATS_STR    (0xC000+(STATS_Y+3)*0x50+(MAP_WIDTH+VIEW_X)*2)
#define VMEM_STATS_DEF    (0xC000+(STATS_Y+4)*0x50+(MAP_WIDTH+VIEW_X)*2)
#define VMEM_STATS_REF    (0xC000+(STATS_Y+5)*0x50+(MAP_WIDTH+VIEW_X)*2)
#define VMEM_STATS_DEX    (0xC000+(STATS_Y+6)*0x50+(MAP_WIDTH+VIEW_X)*2)
#define VMEM_STATS_GOLD   (0xC000+(STATS_Y+8)*0x50+(MAP_WIDTH+VIEW_X)*2)
#define VMEM_DUNGEON_LVL  (0xC000+(STATS_Y+10)*0x50+(MAP_WIDTH+VIEW_X)*2)
#define VMEM_HUNGRY       (0xC000+(STATS_Y+12)*0x50+(MAP_WIDTH+VIEW_X)*2)



// Available actions for player
enum TAction {
  NONE, PLAYER_MOVE, PLAYER_WAIT, NEW_LEVEL
};


#endif