//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
typedef struct TFighter;
typedef struct TAI;
typedef struct TContainer;
typedef struct TItem;


// Entity types
enum Etype {
// PLAYER
  PLAYER,
// MONSTERS
  GOBLIN,
  BAT,
  SNAKE,
  KOBOLD,
  ORC,
// POTIONS
  HEAL_POTION,
  POISON_POTION,
  STR_POTION,
  DEF_POTION,
  DEX_POTION,
  REF_POTION,
//FOOD
  FOOD,
// GOLD
  GOLD,
// ORUTRIX
  ORUTRIX,
// STAIRS
  STAIRSUP,
  STAIRSDOWN,
  n_etypes,

  ERANDOM   // Used to specify we want a random entity
};

#define FIRST_ITEM  HEAL_POTION
#define LAST_ITEM   GOLD
#define FIRST_MOB   GOBLIN
#define LAST_MOB    ORC

struct TEntityInfo {
  enum Etype  type;
  char        *name;
  char        ch;
};

extern const struct TEntityInfo entity_info[];

struct TEntity {
  enum Etype        type;      ///< Type of this entity
  i8                x, y;      ///< Current position
  i8                px, py;    ///< Previous position
  u8                flags;
  u8                xp_level;   ///< Experience level

  struct TFighter   *fighter;   ///< A fighter component for combat
  struct TAI        *ai;        ///< An AI component if intelligent
  struct TContainer *container; ///< Container if carries items
  struct TItem      *item;      ///< Item if pickable
  struct TStairs    *stairs;    ///< For stairs entities
};


extern struct TEntity entities[];
extern u8 num_entities;


void init_entities();
struct TEntity *entity_create       (enum Etype       type,
                                    i8                x, i8 y,
                                    struct TFighter   *fighter,
                                    struct TAI        *ai,
                                    struct TContainer *container,
                                    struct TItem      *item,
                                    struct TStairs    *stairs);

bool            entity_move         (struct TEntity *e, i8 dx, i8 dy);
struct TEntity *get_fighter_at      (i8 x, i8 y);
struct TEntity *get_item_at         (i8 x, i8 y);
struct TEntity *get_stairs_at       (i8 x, i8 y);
bool            is_there_an_entity  (i8 x, i8 y);
void            add_to_world        (struct TEntity *e, i8 x, i8 y);
void            remove_from_world   (struct TEntity *e);
void            check_level         ();
bool            is_potion           (struct TEntity *e);
bool            is_known            (struct TEntity *e);
void            entity_find         (struct TEntity *array[],
                                     struct TEntity *e,
                                     u8 *index);
void            remove_from_list    (struct TEntity *array[],
                                     struct TEntity *e);
void            add_to_list         (struct TEntity *array[],
                                     struct TEntity *e);

inline bool is_dead(struct TEntity *e)
{
  return (e->flags & E_DEAD) != 0;
}
inline bool in_world(struct TEntity *e)
{
  return (e->flags & E_IN_WORLD) != 0;
}
inline void set_dead(struct TEntity *e)
{
  e->flags |= E_DEAD;
}
inline char ech(struct TEntity *e)
{
  return (is_dead(e)) ? '%' : entity_info[e->type].ch;
}
char *name(struct TEntity *e);

void remove_bonus(struct TEntity *target);