//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
#include <cpctelera.h>
#include <stdbool.h>
#include "conio.h"
#include "game.h"
#include "credits.h"
#include "permadeath.h"
#include "consts.h"
#include "components/ai.h"
#include "components/fighter.h"
#include "components/container.h"
#include "components/items.h"
#include "components/rect.h"
#include "components/stairs.h"
#include "game_map.h"
#include "entity.h"
#include "draw.h"
#include "aux_math.h"


u8          turns_poisoned;
u8          turns_bonus;
u8          turns_to_hungry;
extern bool orutrix_found;
u8          orutrix_dlvl;
bool        orutrix_placed;

// Forward declaration private functions;
void _ask_player_name(u8 x, u8 y);

//---------------------------------------------------------------------------
void transition_previous()
{

}
//---------------------------------------------------------------------------
void transition_next()
{
  extern bool fov_changed;
  extern bool stats_changed;
  extern u8 dungeon_lvl;
  log_msg_ext("\477\11\10Going down");
  init_stairs();
  map_create (MAP_WIDTH, MAP_HEIGHT);
  fov_changed = false;
  stats_changed = true;
  dungeon_lvl++;

  // first time need to draw the game manually
  clr_map();
  draw_game(true);
}
//---------------------------------------------------------------------------
void init_all()
{
  extern u16 steps = 0;
  extern u8 dungeon_lvl = 1;
  extern u8 hardest_mob_in_level = SNAKE;
  extern u8 known_potions[];
  turns_poisoned  = 0;
  turns_bonus     = 0;
  turns_to_hungry = HUNGRY_COUNTER;
  orutrix_found   = false;

  // Which dungeon level will the Orutrix be placed
  orutrix_dlvl = rand_range(ORUTRIX_MIN_DLVL, ORUTRIX_MAX_DLVL);
  orutrix_placed = false;

  // Potions are unknown when new game is created
  cpct_memset (known_potions, 0, MAX_POTIONS);

  init_entities();
  init_ai();
  init_fighters();
  init_items();
  init_stairs();
}
//---------------------------------------------------------------------------
void start_game()
{
  cls();
  display_credits();
  _ask_player_name(4, 20);
}
//---------------------------------------------------------------------------
void do_background_actions()
{
  extern u16            steps;
  extern struct TEntity *player;

  steps += 1;

  // Hungry system
  turns_to_hungry--;
  if (!turns_to_hungry) {
    player->flags |= E_HUNGRY;
    draw_player_hungry(false);
  }

  if (player->flags & E_HUNGRY) {
    player->fighter->hp = max (player->fighter->hp - 1, 0);
    draw_player_hp();
  }

  // Every 4th step, restore player's health if not poisoned, or hungry
  if (!(steps&3) && !(player->flags & E_POISONED)
                 && !(player->flags & E_HUNGRY)) {
    player->fighter->hp = min (player->fighter->hp + 1,
                              player->fighter->max_hp);
    draw_player_hp();
  }

  // Take -1hp if poisoned or hungry
  if (player->flags & E_POISONED) {
    if (turns_poisoned != POISON_DURATION) {
      player->fighter->hp = max (player->fighter->hp - 1, 0);
      turns_poisoned++;
      draw_player_hp();
    }
    else {
      player->flags &= ~E_POISONED;
      turns_poisoned = 0;
      log_msg_ext ("\477\11\10You no longer feel poisoned");
    }
  } // if E_POISONED

  // Remove improved stats bonus if time expired
  if (player->flags & E_STATS_IMPR) {
    if (turns_bonus != BONUS_DURATION) {
      turns_bonus++;
    }
    else {
      remove_bonus (player);
      turns_bonus = 0;
    }
  } // if E_STATS_IMPROVED

}

//---------------------------------------------------------------------------
// PRIVATE FUNCTIONS
//---------------------------------------------------------------------------
void _ask_player_name(u8 x, u8 y)
{
  extern char player_name[];
  char **pname = &(entity_info[PLAYER].name);
  *pname = player_name;

  cpct_drawStringM1_f ("Hero's name: ",
    cpct_getScreenPtr (CPCT_VMEM_START, x*2, y*8),
    3,0);

  get_string(player_name, x+14,y, 2,0);
}