//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
#include <cpctelera.h>
#include <stdio.h>
#include <stdbool.h>
#include "consts.h"
#include "components/ai.h"
#include "components/fighter.h"
#include "components/items.h"
#include "components/rect.h"
#include "components/stairs.h"
#include "entity.h"
#include "aux_math.h"
#include "potions.h"
#include "game_map.h"
#include "potions.h"
#ifdef DEBUG
#include <assert.h>
#endif

//
// ITEMS
//
//---------------------------------------------------------------------------
/*!
 * Creates a basic item and return the entity that owns it.
 */
struct TEntity *gen_basic_item (enum ItemType type, i8 x, i8 y)
{
  struct TItem *item = item_create (false);
  struct TEntity *entity = entity_create (type, x, y,
    NULL, NULL, NULL, item, NULL);

 return entity;
}
//---------------------------------------------------------------------------
/*!
 * Generates an item of type item_type and returns the entity that owns it.
 * If item_type is ERANDOM it generates a random item.
 */
struct TEntity *gen_item (enum Etype item_type, i8 x, i8 y)
{
  struct TEntity *entity_item;
  u8 item_info_id;

  if (item_type == ERANDOM) {
    if (cpct_rand () < 80) item_type = FOOD;
    else
      item_type = rand_range (FIRST_ITEM, LAST_ITEM);
  }

  entity_item = gen_basic_item (item_type, x,y);
  item_info_id = item_type - FIRST_ITEM;
  entity_item->item->value  = item_info[item_info_id].value;
  entity_item->item->use_fn = item_info[item_info_id].use_fn;
  entity_item->item->is_gold = (item_type == GOLD);

#ifdef DEBUG
  assert (entity_item != NULL && entity_item->item != NULL);
#endif

  return entity_item;
}
//---------------------------------------------------------------------------
//
// STAIRS
//
/*!
 * Generates a stairs going going to next/previous level
 */
struct TEntity *gen_stairs (i8 x, i8 y, bool downwards)
{
  struct TEntity *e =
  entity_create (STAIRSUP, x,y, NULL,NULL,NULL,NULL,
    stairs_create (downwards));

  e->type = (downwards) ? STAIRSDOWN : e->type;

  return e;
}
//---------------------------------------------------------------------------
//
// ORUTRIX
//
/*!
 * Generates the Orutrix
 */
struct TEntity *gen_orutrix (i8 x, i8 y)
{
  struct TEntity *e = gen_item (ORUTRIX, x,y);
  return e;
}
//---------------------------------------------------------------------------
//
// PLAYER
//
/*!
 * Generates a player of level 1
 */
struct TEntity *gen_player()
{
  struct TEntity *e;
  u8 max_hp, strn, def, ref, dex;
  extern struct TContainer inv_player;

  // Create random stats for player
  max_hp  = 10 + rand_range (1, 10);
  strn    = 9 + rand_range (1, 4);
  def     = 9 + rand_range (1, 4);
  ref     = 9 + rand_range (1, 4);
  dex     = 9 + rand_range (1, 4);
  // Create the player entity with a fighter component
  // Position = 0,0, but it will be changed later when creating the dungeon
  e = entity_create(
      PLAYER,
      0, 0,
      fighter_create(
        max_hp, 0, strn, def, ref, dex, 1, 6,
        kill_player),
      NULL,
      &inv_player,
      item_create (false),
      NULL);

  remove_from_world(e);
  e->xp_level = 1;

  return e;
}
//---------------------------------------------------------------------------
//
// MOB
//
struct TEntity *gen_mob (enum Etype entity_type, i8 x, i8 y)
{
  u8 mob_info_id;
  extern u8 hardest_mob_in_level;
  extern u8 dungeon_lvl;

  if (entity_type == ERANDOM) {
    // every 2 levels create harder mobs
    if (!(dungeon_lvl&3)) {
      hardest_mob_in_level += 1;
      if (hardest_mob_in_level > LAST_MOB) hardest_mob_in_level = LAST_MOB;
    }
    entity_type = rand_range (FIRST_MOB, hardest_mob_in_level);
  }

  mob_info_id = entity_type - FIRST_MOB;

  return entity_create (
    entity_type,
    x,y,
    fighter_create (
      mob_info[mob_info_id].hp,
      mob_info[mob_info_id].xp,
      mob_info[mob_info_id].strn,
      mob_info[mob_info_id].def,
      mob_info[mob_info_id].ref,
      mob_info[mob_info_id].dex,
      mob_info[mob_info_id].min_dmg,
      mob_info[mob_info_id].max_dmg,
      kill_mob
    ),
    basic_ai_create (), NULL, NULL, NULL
  );
}