//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
#include <cpctelera.h>
#include <stdio.h>
#include <stdbool.h>
#include "consts.h"
#include "conio.h"
#include "game.h"
#include "input.h"
#include "draw.h"
#include "entity.h"
#include "components/ai.h"
#include "components/fighter.h"
#include "components/container.h"
#include "components/items.h"
#include "components/rect.h"
#include "components/stairs.h"
#include "game_map.h"
#include "potions.h"
#include "generators.h"
#include "permadeath.h"
#include "orutrix.h"

// globals
const char        player_name[40] = "Thorbag";
const u8          game_palette[] = {
  BLACK, PASTEL_YELLOW, RED, CYAN
};

bool              fov_changed,
                  stats_changed;
struct TEntity    *player;
struct TContainer inv_player;
bool              game_running;
bool              orutrix_found;
u32               global_seed;
u16               steps;
u8                dungeon_lvl;
u8                hardest_mob_in_level;
u8                known_potions[MAX_POTIONS];
u8                msg[38];

//---------------------------------------------------------------------------
void game_loop() {

  i8 dx, dy;
  struct TEntity **pentity;
  struct TAI *ai;
  enum TAction player_action = NONE;

  game_running = true;
  while (game_running) {

    // Handle player input
    dx = dy = 0;
    fov_changed = false;
    player_action = handle_keys(&dx, &dy);

    if (player_action != NONE) clr_log();
    if (player_action == PLAYER_MOVE) {

      // if player bumps into a wall it does not count as a turn
      if (!entity_move (player, dx, dy)) {
        player_action = NONE;
        fov_changed = false;
      }
    } // if (action == PLAYER_MOVE)

    // Calculate fov (only if needed)
    //compute_fov();

    if (player_action != NONE) {
      pentity = last_entity_in_level;//entities + num_entities;
      while(pentity-- != entities_in_level) {
        if ((*pentity)->ai) {
          ai = (*pentity)->ai;
          ai->take_turn(ai);
        } // if (e->ai)
      } // while (e != entities)

      // Do background actions like restore health, calculate hungry
      // etc.
      do_background_actions();

      // check if player's health <= 0
      if (player->fighter->hp <= 0) {
        game_running = false;
      }
    } // if (player_action != NONE)

    // Draw the game
    draw_game(fov_changed);
  } // while (game_running)

  if (orutrix_found)
    happy_ending();
  else
    display_permadeath();

  press_any_key_cont();
  //cpct_reenableFirmware (fw_address);
} // game_loop

//---------------------------------------------------------------------------
void game_init() {

  // The firmware is already disabled from the loader. If you are running
  // this as a standalone project (without the loader), then you
  // MUST uncomment the following line
  cpct_disableFirmware();

  init_all();

  player = gen_player();
  map_create (MAP_WIDTH, MAP_HEIGHT);

  fov_changed = false;
  stats_changed = true;

  // first time need to draw the game manually
  cpct_setPalette (game_palette, 4);
  clr_map();
  draw_game(true);
  sprintf(msg,"\477\11\12Welcome to your doom %s!", name(player));
  log_msg_ext (msg);
}

/*             _______________________________________________
 *    ________|    _     _               __        __   ___   |_______
 *    \       |   / '   / \   |\    /|  |__  |    |  |   |    |      /
 *     \      |   \_.  | - |  |  \/  |  |__  |__  |__|   |    |     /
 *     /      |_______________________________________________|     \
 *    /__________)                                        (__________\
 */
void main(void) {

  while (1) {
    start_game();
    game_init();
    game_loop();
  }

}
