//-----------------------------LICENSE NOTICE--------------------------------
//  This file is part of CPCRogue: An Amstrad CPC rogue like game
//  Copyright (C) 2019 Andrés Mata Bretón (@FlautinesMata)
//
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU Lesser General Public License as published
//  by the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU Lesser General Public License for more details.
//
//  You should have received a copy of the GNU Lesser General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
#include <cpctelera.h>
#include <stdio.h>
#include <stdbool.h>
#include "consts.h"
#include "potions.h"
#include "aux_math.h"
#include "conio.h"
#include "entity.h"
#include "components/fighter.h"
#include "draw.h"

//---------------------------------------------------------------------------
void set_known (u8 potion_id, bool status)
{
  known_potions[potion_id-HEAL_POTION] = status;
}

//---------------------------------------------------------------------------
void cast_heal(struct TEntity *target, u8 value)
{
  extern u8 msg[];
  extern struct TEntity *player;
  u8 heal_value = rand_range (2, value);

  target->fighter->hp = min (target->fighter->hp + heal_value,
                              target->fighter->max_hp);

  sprintf (msg, "\477\11\10%s healed for \477\12\10%d \477\11\10hp",
    name (target), heal_value);
  log_msg_ext (msg);

  if (target == player)
    draw_player_hp();

  set_known (HEAL_POTION, true);
}

//---------------------------------------------------------------------------
void cast_poison (struct TEntity *target, u8 value)
{
  extern u8 msg[];
  extern struct TEntity *player;
  u8 poison_value = rand_range (2, value);

  target->fighter->hp = max (target->fighter->hp - poison_value, 0);
  target->flags |= E_POISONED;

  sprintf (msg, "\477\11\10%s is poisoned and lost \477\12\10%d \477\11\10hp", name (target), poison_value);
  log_msg_ext (msg);

  if (target == player)
    draw_player_hp();

  set_known (POISON_POTION, true);
}

//---------------------------------------------------------------------------
void cast_bonus (struct TEntity *target, u8 value)
{
  extern u8 msg[];
  extern struct TEntity *player;
  void (*draw_stat)();

  // Only one stat can be improved at any time, so removed
  // any augmented stat currently applied
  if (target->flags & E_STATS_IMPR)
    remove_bonus(target);

  switch (value) {
    case 0:   // STR_POTION
      target->fighter->strn+=2;
      target->flags |= E_STR_BONUS;
      sprintf (msg, "\477\11\10You feel stronger");
      set_known (STR_POTION, true);
      draw_stat = draw_player_strn;
      break;

    case 1:  // DEF_POTION
      target->fighter->def+=2;
      target->flags |= E_DEF_BONUS;
      sprintf (msg, "\477\11\10Your defense is improved");
      set_known (DEF_POTION, true);
      draw_stat = draw_player_def;
      break;

    case 2: // DEX_POTION
      target->fighter->dex+=2;
      target->flags |= E_DEX_BONUS;
      sprintf (msg, "\477\11\10You feel nimble as a cat");
      set_known (DEX_POTION, true);
      draw_stat = draw_player_dex;
      break;

    case 3: // REF_POTION
      target->fighter->ref+=2;
      target->flags |= E_REF_BONUS;
      sprintf (msg, "\477\11\10Your reflexes are improved\477\12\10");
      set_known (REF_POTION, true);
      draw_stat = draw_player_ref;
      break;
  }

  log_msg_ext (msg);

  if (target == player)
    draw_stat();
}

//---------------------------------------------------------------------------
void remove_hungry (struct TEntity *target, u8 value)
{
  extern u8 msg[];
  extern u8 turns_to_hungry;

  target->flags &= ~E_HUNGRY;
  turns_to_hungry = HUNGRY_COUNTER;
  log_msg_ext ("\477\11\10You no longer feel hungry");
  draw_player_hungry (true);
}